# -*- coding: utf8 -*-
import os
import unittest

import Repository
from Logger import *

class LoggerTestCase(unittest.TestCase):
	def setUp(self):
		Repository.createRepository()
	def tearDown(self):
		Repository.removeRepository()
	def test_Log(self):
		logDir = Repository.getLogRepository()
		logPath = os.path.join(logDir, 'editLog')
		self.failIf(os.path.exists(logPath))
		log('editLog', 1,2,3)
		self.assert_(os.path.exists(logPath))
		self.assertEquals(file(logPath, 'rb').read(), '1\t2\t3\n')
		log('editLog', 4,5,6)
		self.assertEquals(file(logPath, 'rb').read(), '1\t2\t3\n4\t5\t6\n')
	def test_EditLog(self):
		logDir = Repository.getLogRepository()
		logPath = os.path.join(logDir, editLogFile)
		self.failIf(os.path.exists(logPath))
		editLog(1,2,3)
		self.assert_(os.path.exists(logPath))
		self.assertEquals(file(logPath, 'rb').read(), '1\t2\t3\n')
		editLog(4,5,6)
		self.assertEquals(file(logPath, 'rb').read(), '1\t2\t3\n4\t5\t6\n')
	
	def test_GetRawLines(self):
		s = '''1000\n2000\n3000'''
		for i in range(100):
			editLog('line %s' % i)
		self.assertEquals(getRawLines(editLogFile), ['line %s\n' % i for i in range(100)])
		self.assertEquals(getRawLines(editLogFile, 20), ['line %s\n' % i for i in range(98, 100)])

		# log is not exist
		Repository.removeRepository()
		self.assertEquals(getRawLines(editLogFile), [])
		Repository.createRepository()

if __name__ == '__main__':
	unittest.main(argv=('', '-v'))
