# -*- coding: utf8 -*-
import unittest
import os

import Repository
import Category
import Page
import Permission

from Revision import *

class RevisionTestCase(unittest.TestCase):
	def setUp(self):
		# 1
		Repository.createRepository()

		repositoryPath = Repository.getBackupRepository()
		self.categoryPath = 'Default'
		self.pageName = 'TagList'
		self.timeSeed = 1176676748
		os.mkdir(os.path.join(repositoryPath, self.categoryPath))
		fname = '%s.%s' % (self.pageName, self.timeSeed)
		filePath = os.path.join(repositoryPath, self.categoryPath, fname) 
		self.perms = '111111'
		self.tags = 'Movie, Banana'
		self.contents = 'Girl With a Pearl'
		
		file(filePath, 'wb').write(self.perms+'\n'+self.tags+'\n'+self.contents)
		# 2
		self.page = Page.createPage(Category.Category(), 'pageName', 'text')
		self.page.backup()

	def tearDown(self):
		Repository.removeRepository()
	def test_GetRevTimeStampList(self):
		repositoryPath = Repository.getBackupRepository()
		categoryPath = 'HulaHula'
		pageName = 'TagList'
		L = []
		timeSeed = 1176676748
		os.mkdir(os.path.join(repositoryPath, categoryPath))
		for i in range(10):
			timeStamp = timeSeed - (600 * i)
			fname = '%s.%s' % (pageName, timeStamp)
			filePath = os.path.join(repositoryPath, categoryPath, fname) 
			#L.append(fname)
			file(filePath, 'wb').write('tags\ncontents')
		timeStampList = getRevTimeStampList(categoryPath, pageName)
		self.assertEquals(len(timeStampList), 10)
		for i in range(9, -1):
			timeStamp = timeSeed - (600 * i)
			self.assertEquals(timeStampList[i], str(timeStamp))
		
	def test_Revision(self):
		rev = Revision(self.categoryPath, self.pageName, self.timeSeed)
		self.assertEquals(rev.pageName, self.pageName)
		self.assertEquals(rev.rawText, self.contents)
	
		self.assertRaises(NotExistRevision, Revision, self.categoryPath, self.pageName, 00000000) #notExist

		self.assertRaises(NotExistRevision, Revision, self.categoryPath, self.pageName, '') # request.timeStamp = null

	def test_RevisionFromPageBackup(self):
		L = getRevTimeStampList(self.page.category.categoryPath, self.page.pageName)
		rev = Revision(self.page.category.categoryPath, self.page.pageName, L[0])
		self.assertEquals(rev.rawText, self.page.rawText)

	def test_toRawText(self):
		rev = Revision(self.categoryPath, self.pageName, self.timeSeed)
		self.assertEquals(rev.toRawText(), 'Tags : %s\n%s\n%s' % (self.tags, (len(self.tags)+7)*'=', self.contents))
	
	def test_Perm(self):
		page = Page.createPage(Category.Category(), 'Page', 'rawText', 'Tag', '000000')
		page.backup()
		L = getRevTimeStampList(page.category.categoryPath, page.pageName)
		rev = Revision(page.category.categoryPath, page.pageName, L[0])
		self.assertEquals(rev.perms.toString(), '000000')
		self.assertEquals(rev.curPerms.toString(), '000000')

		page.editPerm('111111')
		rev.load()
		self.assertEquals(rev.perms.toString(), '000000')
		self.assertEquals(rev.curPerms.toString(), '111111')


		page.remove()
		rev.loadCurPerm()
		self.assertEquals(rev.perms.toString(), '000000')
		self.assertEquals(rev.curPerms.toString(), '000000')
	def test_GetLastTimeStamp(self):
		timeStamp = getLastTimeStamp(self.categoryPath, self.pageName)
		self.assertEquals(timeStamp, self.timeSeed)



if __name__ == '__main__':
	unittest.main(argv=('', '-v'))
