# -*- coding: utf8 -*-
import unittest
from Session import *

class SessionTestCase(unittest.TestCase):
	def setUp(self):
		Repository.createRepository()
		self.sess = Session()

	def tearDown(self):
		Repository.removeRepository()

	def test_CreateSession(self):
		sess = Session()
		self.assertEquals(sess.data, {})
		self.assert_(sess.sessionPath.endswith(sess.sid))

		sess2 = Session()
		self.assertNotEquals(sess.sid, sess2.sid)

	def test_SessionDump(self):
		self.sess['uid'] = 'jangc'
		self.assertEquals(self.sess['uid'], 'jangc')

	def test_LoadSession(self): 
		self.sess['name'] = 'jang'
		loadedSess = Session(self.sess.sid)
		self.assertEquals(loadedSess.sid, self.sess.sid)
		self.assertEquals(loadedSess['name'], 'jang')

	def test_IsLive(self):
		sess = Session()
		self.assertEquals(sess.isLive(), False)
		sess['uid'] = 'jangc'
		self.assertEquals(sess.isLive(), True)

		orgLife = Session.sessionLife
		Session.sessionLife = 0
		self.assertEquals(sess.isLive(), False)
		Session.sessionLife = orgLife

	def test_Remove(self):
		sess = Session()
		self.assertEquals(sess.isLive(), False)
		sess['uid'] = 'jangc'
		self.assertEquals(sess.isLive(), True)
		self.assertEquals(sess['uid'], 'jangc')
		sess.remove()
		self.assertEquals(sess.isLive(), False)
		self.assertRaises(KeyError, sess.__getitem__, 'uid')

if __name__ == '__main__':
	unittest.main(argv=('', '-v'))
