# -*- coding: utf8 -*
import unittest
import os

import Repository
import Page
import Category
from TagManager import *
from Tag import *


class TagManagerTestCase(unittest.TestCase):
	def setUp(self):
		Repository.createRepository()
		self.category = Category.Category()

	def tearDown(self):
		Repository.removeRepository()

	def test_Init(self):
		tags = 'Movie, Leo'
		tm = TagManager(tags)
		self.assertEquals(tm.tagSet, sets.Set(['Movie', 'Leo']))

		tags = 'Movie, Leo, Leo'
		tm = TagManager(tags)
		self.assertEquals(tm.tagSet, sets.Set(['Movie', 'Leo']))

	def test_GetTagString(self):
		tags = 'Movie, Leo'
		tm = TagManager(tags)
		self.assertEquals(tm.getTagString(), tags)

	def test_diff(self):
		tags1 = 'Movie, Leo'
		tm = TagManager(tags1)

		tags2 = 'Movie, Dicafrio'
		added, removed = tm.diff(tags2)
		self.assertEquals(added, ['Dicafrio'])
		self.assertEquals(removed, ['Leo'])

	def test_Update(self):
		pageName = 'yobi'
		page = Page.createPage(self.category, pageName, '')
		tags = 'Movie, Leo'
		tagList = filter(None, map(lambda obj:obj.strip(), tags.split(',')))
		for tag in tagList:
			Tag(tag).append(page)
			path = os.path.join(Repository.getTagRepository(), tag)
			self.assert_(os.path.isfile(path))
		

		tm = TagManager(tags)

		newTags = 'Movie, Dicafrio'
		added, removed = tm.diff(newTags) # for test
		tm.update(newTags, page)
		for tag in added:
			path = os.path.join(Repository.getTagRepository(), tag)
			self.assert_(os.path.isfile(path))
			self.assert_(pageName in file(path, 'rb').read().split('\n'))
		for tag in removed: # FIXME 실행되고있지 않음.
			path = os.path.join(Repository.getTagRepository(), tag)
			if os.path.isfile(path):
				self.failIf(pageName in file(path, 'rb').read().split('\n'))
		
if __name__ == '__main__':
	unittest.main(argv=('', '-v'))
