# -*- coding: utf8 -*-
import unittest
import sys
sys.path.append('..') 

from imFormatter.HTMLFormatter import HTMLFormatter

class HTMLFormatterTestCase(unittest.TestCase):
	def test_TableFormatting(self):
		formatter = HTMLFormatter()
		self.assertEquals(formatter.table([['1']]), '<table><tr><td>1</td></tr></table>')
		self.assertEquals(formatter.table([['1', '2']]), '<table><tr><td>1</td><td>2</td></tr></table>')
		self.assertEquals(formatter.table([['1'], ['2']]), '<table><tr><td>1</td></tr><tr><td>2</td></tr></table>')

		self.assertEquals(formatter.table([[('left', '1')], ['2']]), '<table><tr><td align="left">1</td></tr><tr><td>2</td></tr></table>')
		self.assertEquals(formatter.table([[('right', '1')], ['2']]), '<table><tr><td align="right">1</td></tr><tr><td>2</td></tr></table>')
		self.assertEquals(formatter.table([[('center', '1')], ['2']]), '<table><tr><td align="center">1</td></tr><tr><td>2</td></tr></table>')

	def test_IndentFormatting(self):
		formatter = HTMLFormatter()
		self.assertEquals(formatter.indent([(1, 'line1')]), '<div class="imIndent">line1</div>')

		self.assertEquals(formatter.indent([(1, 'line1'), (2, 'line2'), (-1, 'line3')]), '<div class="imIndent">line1<div class="imIndent"><div class="imIndent">line2</div>line3</div></div>')

		self.assertEquals(formatter.indent([(1, 'line1\n'), (-1, '\n'), (1, 'line2\n')]), '<div class="imIndent">line1\n</div>\n<div class="imIndent">line2\n</div>')

	def test_List(self):
		formatter = HTMLFormatter()
		L = [(1, None, 'line1')]
		self.assertEquals(formatter.List(L), '<ul>line1</ul>')

		L = [(1, '1', 'line1')]
		self.assertEquals(formatter.List(L), '<ol type="1">line1</ol>')

		L = [(1, '1', 'line1'), (1, 'a', 'line2')]
		self.assertEquals(formatter.List(L), '<ol type="1">line1<ol type="a">line2</ol></ol>')

		L = [(1, None, 'line1'), (1, 'a', 'line2')]
		self.assertEquals(formatter.List(L), '<ul>line1<ol type="a">line2</ol></ul>')

		L = [(1, None, 'line1'), (1, 'a', 'line2'), (-1, 'i', 'line3')]
		self.assertEquals(formatter.List(L), '<ul>line1<ol type="a">line2</ol>line3</ul>')

		L = [(1, None, 'line1'), (1, 'a', 'line2'), (1, 'i', 'line3'), (-2, None, 'line4'), (1, 'I', 'line5')]
		self.assertEquals(formatter.List(L), '<ul>line1<ol type="a">line2<ol type="i">line3</ol></ol>line4<ol type="I">line5</ol></ul>')
			 

if __name__ == '__main__':
	unittest.main(argv=('', '-v'))
