# -*- coding: utf8 -*-
import unittest

from UserDB import *
import Repository

class UserDBTestCase(unittest.TestCase):
	def setUp(self):
		Repository.createRepository()
	def tearDown(self):
		Repository.removeRepository()
	def test_IsExistUser(self):
		udb = UserDB()
		self.assertEquals(udb.isExistUser('uid'), False)
		self.assert_(udb.addUser('uid', 'password', 'e@mail'))
		self.assertEquals(udb.isExistUser('uid'), True)
	def test_AddUser(self):
		udb = UserDB()
		user = udb.addUser('uid', 'password', 'e@mail')
		self.assert_(isinstance(user, User.User))
		self.assertRaises(AlreadyExistUser, udb.addUser, 'uid', 'password', 'e@mail')
	def test_GetUidList(self):
		udb = UserDB()
		self.assertEquals(udb.getUidList(), [])
		user = udb.addUser('uid', 'password', 'e@mail')
		self.assertEquals(udb.getUidList(), ['uid'])
		user = udb.addUser('uid2', 'password', 'e@mail')
		uidList = udb.getUidList()
		uidList.sort()
		self.assertEquals(uidList, ['uid', 'uid2'])
	def test_GetUser(self):
		udb = UserDB()
		uid = 'uid'
		self.assert_(udb.addUser(uid, 'password', 'e@mail'))
		user = udb.getUser(uid, 'password')
		self.assertEquals(user.uid, uid)

		self.assertRaises(NotCorrectPassword, udb.getUser, uid, 'invalidPW')




if __name__ == '__main__':
	unittest.main(argv=('', '-v'))
