# -*- coding: utf8 -*-
import unittest
import sys
sys.path.append('..')

from imParser import WikiParser


class IndentParserTestCase(unittest.TestCase):
	def test_prsing(self):
		indentParser = WikiParser.IndentParser()
		
		indentParser.feed(""" line1""")
		self.assertEquals(indentParser.parse(), [(1, 'line1\n')])

		indentParser.feed(""" line1\n""")
		self.assertEquals(indentParser.parse(), [(1, 'line1\n')])

		indentParser.feed(""" line1\n line2""")
		self.assertEquals(indentParser.parse(), [(1, 'line1\nline2\n')])

		indentParser.feed(""" line1\n  line2\n   line3""")
		self.assertEquals(indentParser.parse(), [(1, 'line1\n'), (1, 'line2\n'), (1, 'line3\n')])

		indentParser.feed(""" line1\n  line2\n line3""")
		self.assertEquals(indentParser.parse(), [(1, 'line1\n'), (1, 'line2\n'), (-1, 'line3\n')])

		indentParser.feed(""" line1\n  line2\n   line3\n line4""")
		self.assertEquals(indentParser.parse(), [(1, 'line1\n'), (1, 'line2\n'), (1, 'line3\n'), (-2, 'line4\n')])

		indentParser.feed(""" line1\n  line2\n   line3\n line4\n   line5""")
		self.assertEquals(indentParser.parse(), [(1, 'line1\n'), (1, 'line2\n'), (1, 'line3\n'), (-2, 'line4\n'), (1, 'line5\n')])

		indentParser.feed(""" line1\n line2\n  line3\n  line4""")
		self.assertEquals(indentParser.parse(), [(1, 'line1\nline2\n'), (1, 'line3\nline4\n')])


	def test_parsing2(self): # 여러개의 \n 처리.
		indentParser = WikiParser.IndentParser()
		indentParser.feed(""" line1\n line2\n\n\n line3""")
		self.assertEquals(indentParser.parse(), [(1, 'line1\nline2\n\n\nline3\n')])
	def test_unbalance(self):
		indentParser = WikiParser.IndentParser()
		indentParser.feed("""   line1\n  line2\n line3""")
		self.assertEquals(indentParser.parse(), [(1, 'line1\n'), (-1, 'line2\n'), (0, 'line3\n')])

class TableParserTestCase(unittest.TestCase):
	def test_multiCellParsing(self):
		tableParser = WikiParser.TableParser()
		
		tableParser.feed("""|| im wiki ||""")
		self.assertEquals(tableParser.parse(), [[' im wiki ']])

		tableParser.feed("""|| im wiki ||   """)
		self.assertEquals(tableParser.parse(), [[' im wiki ']])

		tableParser.feed("""|| im wiki || second cell ||""")
		self.assertEquals(tableParser.parse(), [[' im wiki ', ' second cell ']])

		tableParser.feed("""|| im wiki || second cell ||
|| egg || kimbab ||""")
		self.assertEquals(tableParser.parse(), [[' im wiki ', ' second cell '], [' egg ', ' kimbab ']])
	def test_align(self):
		tableParser = WikiParser.TableParser()
		
		tableParser.feed("""||< im wiki ||""")
		self.assertEquals(tableParser.parse(), [[('left', ' im wiki ')]])

		tableParser.feed("""||> im wiki ||""")
		self.assertEquals(tableParser.parse(), [[('right', ' im wiki ')]])

		tableParser.feed("""||<> im wiki ||""")
		self.assertEquals(tableParser.parse(), [[('center', ' im wiki ')]])

		tableParser.feed("""||>< im wiki ||""")
		self.assertEquals(tableParser.parse(), [[('center', ' im wiki ')]])

class wikiTestCase(unittest.TestCase):
	def test_feed(self):
                rawText = 'imwiki'
		parser = WikiParser.Parser(WikiParser.EchoFormatter())
		parser.feed(rawText)
		self.assertEquals(parser.rawText, rawText)
		
	def test_strong(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())

		parser.feed("'''imwiki'''")
		self.assertEquals(parser.parse(), "<strong>imwiki</strong>")

		parser.feed("im'''wiki'''")
		self.assertEquals(parser.parse(), "im<strong>wiki</strong>")

		parser.feed("''''imwiki'''")
		self.assertEquals(parser.parse(), "'<strong>imwiki</strong>")

		parser.feed("''''''imwiki'''")
		self.assertEquals(parser.parse(), "'''<strong>imwiki</strong>")

		parser.feed("''''''im'wiki'''")
		self.assertEquals(parser.parse(), "'''<strong>im'wiki</strong>")

		parser.feed("'''im'\nwiki'''")
		self.assertEquals(parser.parse(), "<strong>im'\nwiki</strong>")

		parser.feed("''' im'wiki'''")
		self.assertEquals(parser.parse(), "''' im'wiki'''")

	def test_em(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())

		parser.feed("''imwiki''")
		self.assertEquals(parser.parse(), "<em>imwiki</em>")

		parser.feed("''imwiki'''")
		self.assertEquals(parser.parse(), "<em>imwiki</em>'")

	def test_strong_em(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())

		parser.feed("'''''imwiki'''''")
		self.assertEquals(parser.parse(), "<strongEm>imwiki</strongEm>")

		parser.feed("'''''imwiki'''''kiwi'''''")
		self.assertEquals(parser.parse(), "<strongEm>imwiki</strongEm>kiwi'''''")

		parser.feed("''im'''''''imwiki'''''kiwi'''")
		self.assertEquals(parser.parse(), "<em>im</em><strongEm>imwiki</strongEm>kiwi'''")

		parser.feed("''im\nkiwi''")
		self.assertEquals(parser.parse(), "<em>im\nkiwi</em>")

	def test_strike(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())

		parser.feed("~~imwiki~~")
		self.assertEquals(parser.parse(), "<strike>imwiki</strike>")

		parser.feed("~~ imwiki~~")
		self.assertEquals(parser.parse(), "~~ imwiki~~")

		parser.feed("~~~imwiki~~~")
		self.assertEquals(parser.parse(), "~<strike>imwiki</strike>~")

		parser.feed("~~im\nwiki~~")
		self.assertEquals(parser.parse(), "<strike>im\nwiki</strike>")

	
	def test_horizon(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())

		parser.feed("----")
		self.assertEquals(parser.parse(), "<horizon>----</horizon>")
		
		parser.feed("-------")
		self.assertEquals(parser.parse(),  "<horizon>-------</horizon>")

		parser.feed("---- ")
		self.assertEquals(parser.parse(), "---- ")

		parser.feed("----\n")
		self.assertEquals(parser.parse(),  "<horizon>----\n</horizon>")
	
	def test_underline(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())

		parser.feed("__imwiki__")
		self.assertEquals(parser.parse(), "<underline>imwiki</underline>")

		parser.feed("___imwiki___")
		self.assertEquals(parser.parse(), "_<underline>imwiki</underline>_")

		parser.feed("__im\nwiki__")
		self.assertEquals(parser.parse(), "<underline>im\nwiki</underline>")


	def test_heading(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())

		parser.feed("= imwiki =")
		self.assertEquals(parser.parse(), "<heading>imwiki : 1</heading>")

		parser.feed("===== imwiki ====")
		self.assertEquals(parser.parse(), "===== imwiki ====")

		parser.feed("===== imwiki =====")
		self.assertEquals(parser.parse(), "<heading>imwiki : 5</heading>")

		parser.feed("======= imwiki =======")
		self.assertEquals(parser.parse(), "======= imwiki =======")

		parser.feed("x== imwiki ==")
		self.assertEquals(parser.parse(), "x== imwiki ==")
		
		parser.feed("== imwiki == x")
		self.assertEquals(parser.parse(), "== imwiki == x")

		parser.feed("=== imwiki ===\n") 
		self.assertEquals(parser.parse(), "<heading>imwiki : 3</heading>")

		parser.feed("=== imwiki ===\n\n\n") 
		self.assertEquals(parser.parse(), "<heading>imwiki : 3</heading><br /><br />")


		parser.feed("=== im\nwiki ===") 
		self.assertEquals(parser.parse(), "=== im<br />wiki ===")

		#parser.feed(" === imwiki ===") # FIXME 
		#self.assertEquals(parser.parse(), "<heading>imwiki</heading>")


		# depth test
		parser.feed("""
'''I'm very strong!!!!'''
=== I have three head! ===
""")
		self.assertEquals(parser.parse(), """<br /><strong>I'm very strong!!!!</strong><br /><heading>I have three head! : 3</heading>""")

	def test_deepReplace(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())

		parser.feed("'''__Im Wiki__'''")
		self.assertEquals(parser.parse(), "<strong><underline>Im Wiki</underline></strong>")
		parser.feed("'''__Im Wiki'''__")
		self.assertEquals(parser.parse(), "<strong>__Im Wiki</strong>__")
		
		parser.feed("=== '''Imwiki''' ===")

	def test_shallowReplace(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())

		parser.feed("'''== Im Wiki =='''")
		self.assertEquals(parser.parse(), "<strong>== Im Wiki ==</strong>")
	def test_pre(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())

		parser.feed("""
{{{
== im ==
'''wiki'''
}}}
""")
		self.assertEquals(parser.parse(), """<br /><pre>
== im ==
'''wiki'''
</pre><br />""")

		parser.feed("""
{{{
== im ==
'''wiki'''
}}}
normal line
{{{
''egg''
}}}""")

		self.assertEquals(parser.parse(), """<br /><pre>
== im ==
'''wiki'''
</pre><br />normal line<br /><pre>
''egg''
</pre>""")
		
	def test_table(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())

		parser.feed("|| egg ||")
		self.assertEquals(parser.parse(), """<table>[[' egg ']]</table>""")

		parser.feed("|| egg || spam ||")
		self.assertEquals(parser.parse(), """<table>[[' egg ', ' spam ']]</table>""")

		parser.feed("|| egg || spam ||\n|| 2 || 3 ||")
		self.assertEquals(parser.parse(), """<table>[[' egg ', ' spam '], [' 2 ', ' 3 ']]</table>""")

		parser.feed("|| egg || spam ||\n||||||")
		self.assertEquals(parser.parse(), """<table>[[' egg ', ' spam '], ['', '']]</table>""")

		parser.feed("|| egg || spam ||\n||||")
		self.assertEquals(parser.parse(), """<table>[[' egg ', ' spam '], ['']]</table>""")

		parser.feed("""|| Python ||
 || egg ||
 || ham ||
""")
		self.assertEquals(parser.parse(), """<table>[[' Python ']]</table><indent>[(1, "<table>[[' egg '], [' ham ']]</table>")]</indent>""")

	def test_listitem(self): # FIXME
		parser = WikiParser.Parser(WikiParser.EchoFormatter())

		parser.feed("""* ham""")
		self.assertEquals(parser.parse(), "* ham")

		parser.feed(" * ham")
		self.assertEquals(parser.parse(), "<List>[(1, None, '<listitem>ham</listitem>\\n')]</List>")

		parser.feed(" * ham\n * egg")
		self.assertEquals(parser.parse(), "<List>[(1, None, '<listitem>ham</listitem>\\n<listitem>egg</listitem>\\n')]</List>") # FIXME \\n?

		parser.feed(" a. ham\n a. egg")
		self.assertEquals(parser.parse(), "<List>[(1, 'a', '<listitem>ham</listitem>\\n<listitem>egg</listitem>\\n')]</List>")

		parser.feed(" A. ham\n A. egg")
		self.assertEquals(parser.parse(), "<List>[(1, 'A', '<listitem>ham</listitem>\\n<listitem>egg</listitem>\\n')]</List>")


		parser.feed(" 1. ham\n 10. egg")
		self.assertEquals(parser.parse(), "<List>[(1, '1', '<listitem>ham</listitem>\\n<listitem>egg</listitem>\\n')]</List>")


	def test_Indent(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())

		parser.feed("line1\n line2")
		self.assertEquals(parser.parse(), "line1<br /><indent>[(1, 'line2<br />')]</indent>")

		parser.feed("line1\n line2\n  line3")
		self.assertEquals(parser.parse(), "line1<br /><indent>[(1, 'line2<br />'), (1, 'line3<br />')]</indent>")

		parser.feed("line1\n line2\n  line3\n line4")
		self.assertEquals(parser.parse(), "line1<br /><indent>[(1, 'line2<br />'), (1, 'line3<br />'), (-1, 'line4<br />')]</indent>")

		parser.feed("line1\n line2\n  line3\n\nline4") #  FIXME
		self.assertEquals(parser.parse(), "line1<br /><indent>[(1, 'line2<br />'), (1, 'line3<br /><br />')]</indent>line4")

		parser.feed("line1\n line2\n  line3\nline4")
		self.assertEquals(parser.parse(), "line1<br /><indent>[(1, 'line2<br />'), (1, 'line3<br />')]</indent>line4")

		parser.feed(" '''line1'''")
		self.assertEquals(parser.parse(), """<indent>[(1, '<strong>line1</strong><br />')]</indent>""") 

		parser.feed(" line1\n * line2")
		self.assertEquals(parser.parse(), """<indent>[(1, 'line1<br /><listitem>line2</listitem><br />')]</indent>""")  # FIXME listitem엔 br 안붙게
		#self.assertEquals(parser.parse(), """<indent>[(1, 'line1<br />')]</indent><List>[(1, '<listitem>line2</listitem>')]</List>""") 

		parser.feed(""" line1\n line2\n\n line3""")
		self.assertEquals(parser.parse(), """<indent>[(1, 'line1<br />line2<br /><br />line3<br />')]</indent>""")  
	
	def test_List(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())

		parser.feed(" * line1")
		self.assertEquals(parser.parse(), "<List>[(1, None, '<listitem>line1</listitem>\\n')]</List>")

		parser.feed(" * line1\n * line2")
		self.assertEquals(parser.parse(), "<List>[(1, None, '<listitem>line1</listitem>\\n<listitem>line2</listitem>\\n')]</List>")

		parser.feed(" * line1\n * line2\nline3")
		self.assertEquals(parser.parse(), "<List>[(1, None, '<listitem>line1</listitem>\\n<listitem>line2</listitem>\\n')]</List>line3")

		parser.feed(" 1. line1\n 1. line2\nline3")
		self.assertEquals(parser.parse(), "<List>[(1, '1', '<listitem>line1</listitem>\\n<listitem>line2</listitem>\\n')]</List>line3")

		parser.feed(" 1. line1\n 1. line2\n line3")
		self.assertEquals(parser.parse(), "<List>[(1, '1', '<listitem>line1</listitem>\\n<listitem>line2</listitem>\\n')]</List><indent>[(1, 'line3<br />')]</indent>")

	def test_ListType(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())
		parser.feed(" * line1")
		self.assertEquals(parser.parse(), "<List>[(1, None, '<listitem>line1</listitem>\\n')]</List>")

		parser.feed(" 1. line1\n a. line2")
		self.assertEquals(parser.parse(), "<List>[(1, '1', '<listitem>line1</listitem>\\n<listitem>line2</listitem>\\n')]</List>")

		parser.feed(" a. line1\n 1. line2")
		self.assertEquals(parser.parse(), "<List>[(1, 'a', '<listitem>line1</listitem>\\n<listitem>line2</listitem>\\n')]</List>")

		parser.feed(" i. line1")
		self.assertEquals(parser.parse(), "<List>[(1, 'i', '<listitem>line1</listitem>\\n')]</List>")

		parser.feed(" b. line1")
		self.assertEquals(parser.parse(), "<List>[(1, 'a', '<listitem>line1</listitem>\\n')]</List>")

	def test_tt(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())
		parser.feed("Im{{{''Wiki''}}}.")
		self.assertEquals(parser.parse(), "Im<tt>''Wiki''</tt>.")

	def test_InternalLink(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())
		parser.feed("Im[Wiki].")
		self.assertEquals(parser.parse(), 'Im<internalLink> : Wiki : False</internalLink>.')

		parser.feed("Im[Wiki]da.")
		self.assertEquals(parser.parse(), 'Im<internalLink> : Wiki : False</internalLink>da.')


		parser.feed("Im[[Wiki]da.")
		self.assertEquals(parser.parse(), 'Im[<internalLink> : Wiki : False</internalLink>da.')

		parser.feed("Im[Seed/Wiki]da.")
		self.assertEquals(parser.parse(), 'Im<internalLink>Seed : Wiki : False</internalLink>da.')

		parser.feed("Im[Default/Seed/Wiki]da.")
		self.assertEquals(parser.parse(), 'Im<internalLink>Default/Seed : Wiki : False</internalLink>da.')

		parser.feed("Im[http://jangc.net]da.")
		self.assertEquals(parser.parse(), 'Im<externalLink>http://jangc.net</externalLink>da.')

		parser.feed("Im[http://jangc.net JANGC Homepage]da.")
		self.assertEquals(parser.parse(), 'Im<externalLink>http://jangc.net : JANGC Homepage</externalLink>da.')

	def test_CamelCase(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())
		parser.feed("Im FrontPage.")
		self.assertEquals(parser.parse(), 'Im <internalLink> : FrontPage : False</internalLink>.')

		parser.feed("Im /FrontPage.")
		self.assertEquals(parser.parse(), 'Im <internalLink> : FrontPage : True</internalLink>.')

		parser.feed("Im CateGory/FrontPage.")
		self.assertEquals(parser.parse(), 'Im <internalLink>CateGory : FrontPage : False</internalLink>.')

		parser.feed("Im CateGory/SubCategory/FrontPage.")
		self.assertEquals(parser.parse(), 'Im <internalLink>CateGory/SubCategory : FrontPage : False</internalLink>.')

		parser.feed("Im /CateGory/SubCategory/FrontPage.") # FIXME parsing된 결과물도 "/"Category인게 맞나?
		self.assertEquals(parser.parse(), 'Im <internalLink>/CateGory/SubCategory : FrontPage : True</internalLink>.')

	def test_ExternalLink(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())
		parser.feed("http://jangc.net")
		self.assertEquals(parser.parse(), '<externalLink>http://jangc.net</externalLink>')
		parser.feed("im http://jangc.net?action=view da")
		self.assertEquals(parser.parse(), 'im <externalLink>http://jangc.net?action=view</externalLink> da')

	def test_Macro(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())
		parser.feed("Im[[Wiki]].")
		self.assertEquals(parser.parse(), "Im<macro>Wiki</macro>.")



if __name__ == '__main__':
	unittest.main(argv=('', '-v'))
