# -*- coding: utf8 -*-
import unittest
import os

import utils
import Repository 
import Category
from Request import Request
import UserDB
import sys
sys.path.append('..')
from webInterface import *

class ActionTestCase(unittest.TestCase):
	def setUp(self):
		Repository.createRepository()

		templateProcessor.setTemplateFolder('../skin/default')
		os.environ['SCRIPT_NAME'] = 'imwiki.cgi'
		request = Request.Request()
		request.init()

		self.categoryPath = ''
		self.pageName = 'FrontPage'
		self.rawText = 'Hello World'
		self.page = Page.createPage(Category.Category(), self.pageName, self.rawText)
		self.page.backup()

	def tearDown(self):
		Repository.removeRepository()
	def test_WriteForm(self):
		request = Request.Request()
		response = do_writeForm(request)
		self.assert_(response.__repr__())
	def test_Write(self):
		request = Request.Request()
		request.pageName = 'Test Page'
		request.categoryPath = ''
		request.rawText = 'text'
		request.user = UserDB.UserDB().addUser('jangc', 'passwd', 'email@email.com')

		request.memberRead, request.memberWrite, request.groupRead, request.groupWrite, request.anonyRead, request.anonyWrite = [True]*6
		response = do_write(request)
		self.assert_(response.__repr__())

		category = Category.Category()
		encPageName = utils.encode(request.pageName)
		self.assert_(os.path.isfile(os.path.join(category.dirPath, encPageName)))

		request.pageName = 'Test Page2'
		request.categoryPath = 'Foods'
		request.memberRead, request.memberWrite, request.groupRead, request.groupWrite, request.anonyRead, request.anonyWrite = ['ok']*6
		response = do_write(request)
		self.assert_(response.__repr__())
		category = Category.getCategoryByCategoryPath(request.categoryPath)
		encPageName = utils.encode(request.pageName)
		self.assert_(os.path.isfile(os.path.join(category.dirPath, encPageName)))
	def test_edit(self):
		request = Request.Request()
		request.tags = 'Test, Tag'
		request.rawText = 'New Text'
		request.mtime = '9999999999'
		request.pageName = self.page.pageName
		request.categoryPath = self.page.category.categoryPath
		request.user = UserDB.UserDB().addUser('jangc', 'passwd', 'email@email.com')
		response = do_edit(request)
		self.assert_(response.__repr__())

	def test_editForm(self):
		request = Request.Request()
		response = do_editForm(request)
		self.assert_(response.__repr__())
	def test_View(self):
		request = Request.Request()
		user = UserDB.UserDB().addUser('jangc', 'passwd', 'email@email.com')
		request.user = user
		response = do_view(request)
		self.assert_(response.__repr__())
	def test_TitleSearch(self):	
		request = Request.Request()
		response = do_titleSearch(request)
		self.assert_(response.__repr__())
	def test_Raw(self):
		request = Request.Request()
		request.categoryPath = ''
		request.pageName = self.pageName
		request.timeStamp = Revision.getRevTimeStampList(self.categoryPath, self.pageName)[0]
		user = UserDB.UserDB().addUser('jangc', 'passwd', 'email@email.com')
		request.user = user
		response = do_raw(request)
		self.assert_(response.__repr__())
	def test_fullSearch(self):
		request = Request.Request()
		request.needle = 'needle'
		response = do_fullSearch(request)
		self.assert_(response.__repr__())
	def test_info(self):
		request = Request.Request()
		response = do_info(request)
		self.assert_(response.__repr__())
	def test_pageListInTag(self):
		request = Request.Request()
		request.tagName = 'Tag'
		response = do_pageListInTag(request)
		self.assert_(response.__repr__())
	def test_remove(self):
		request = Request.Request()
		request.pageName = self.pageName
		response = do_remove(request)
		self.assert_(response.__repr__())
	def test_join(self):
		request = Request.Request()
		request.sess = {}
		request.uid = 'jangc'
		request.passwd = 'passwd'
		request.passwdConfirm = 'passwd'
		request.email = 'email@email.com'
		response = do_join(request)
		self.assert_(response.__repr__())
	def test_login(self):
		user = UserDB.UserDB().addUser('jangc', 'passwd', 'email@email.com')
		request = Request.Request()
		request.sess = {}
		request.uid = 'jangc'
		request.passwd = 'passwd'
		response = do_login(request)
		self.assert_(response.__repr__())

	def test_logout(self):
		class MockSession(dict):
			def remove(self):pass
		request = Request.Request()
		request.sess = MockSession()
		response = do_logout(request)
		self.assert_(response.__repr__())

	def xtest_updateUser(self): # FIXME webInferface에서 raise를 사용하면 그대로, 그렇지 않으면 고쳐줘야 함. 안돌아감.
		request = Request.Request()
		request.user = User.User('anonymous')
		self.assertRaises(InvalidUserId, do_updateUser, request)

		request.user = UserDB.UserDB().addUser('jangc', 'passwd', 'email@email.com')
		request.passwd = 'passwd'
		request.passwdConfirm = ''
		self.assertRaises(CheckYourPassword, do_updateUser, request)

		request.passwdConfirm = request.passwd
		request.email = 'email.com'
		self.assertRaises(CheckYourEmail, do_updateUser, request)

		request.email = 'email@email.com'
		response = do_updateUser(request)
		self.assert_(response.__repr__())

if __name__ == '__main__':
	unittest.main(argv=('', '-v'))
