# -*- coding: utf8 -*-
import unittest
import sys
sys.path.append('..')

import Repository
import Category
import Page
from imMacro import TitleIndex


class TitleIndexTestCase(unittest.TestCase):
	def setUp(self):
		Repository.createRepository()
	def tearDown(self):
		Repository.removeRepository()
	def test_CategorizingByIndexKey(self):
		pageList = Category.Category().getPageListFromDescendants()
		D = TitleIndex.categorizingByIndexKey(pageList)
		self.assertEquals(len(D), 0) 

		Page.createPage(Category.Category(), 'Air', 'text')
		Page.createPage(Category.Category(), 'abc', 'text')
		pageList = Category.Category().getPageListFromDescendants()
		D = TitleIndex.categorizingByIndexKey(pageList)
		self.assertEquals(D.keys(), ['A'])

		Page.createPage(Category.Category(), 'Best', 'text')
		pageList = Category.Category().getPageListFromDescendants()
		D = TitleIndex.categorizingByIndexKey(pageList)
		self.assertEquals(D.keys(), ['A', 'B'])

		Page.createPage(Category.Category(), '감귤', 'text')
		pageList = Category.Category().getPageListFromDescendants()
		D = TitleIndex.categorizingByIndexKey(pageList)
		L = D.keys()
		L.sort()
		self.assertEquals(L, ['A', 'B', '가'])


		Page.createPage(Category.Category(), '*$', 'text')
		pageList = Category.Category().getPageListFromDescendants()
		D = TitleIndex.categorizingByIndexKey(pageList)
		L = D.keys()
		L.sort()
		self.assertEquals(L, ['A', 'B', 'Others', '가'])

		Page.createPage(Category.Category(), 'ㅋㅋㅋ', 'text')
		pageList = Category.Category().getPageListFromDescendants()
		D = TitleIndex.categorizingByIndexKey(pageList)
		L = D.keys()
		L.sort()
		self.assertEquals(L, ['A', 'B', 'Others', '가'])





if __name__ == '__main__':
	unittest.main(argv=('', '-v'))
