# -*- coding: utf8 -*-
import unittest
import os
import shutil

from templateProcessor import *

class TemplateProcessorTestCase(unittest.TestCase):
	def setUp(self):
		self.testFolder = 'test'
		try: shutil.rmtree(self.testFolder)
		except: pass

		os.makedirs(self.testFolder)
		setTemplateFolder('')

	def tearDown(self):
		shutil.rmtree(self.testFolder)

		
	def test_SetTemplateFolder(self):
		fileName = 'test.template'
		contents = 'more ham plz~'
		self.assertRaises(IOError, getTemplate, fileName)
		file(os.path.join(self.testFolder, fileName), 'wb').write(contents)
		setTemplateFolder(self.testFolder)
		self.assertEquals(getTemplate(fileName), contents)

	def test_GetTemplate(self):
		testFile = 'test2.template'
		contents = 'Positioning'
		filePath = os.path.join(self.testFolder, testFile)
		file(filePath, 'wb').write(contents)
		template = getTemplate(filePath)
		self.assertEquals(template, contents)
		os.remove(filePath)

		

	def test_Evaluate(self):
		template = '<:100:>'
		self.assertEquals(templateProcessor(template), "100")
		template = '<:"Wiki":>'
		self.assertEquals(templateProcessor(template), "Wiki")

		template = '12<:100:>12'
		self.assertEquals(templateProcessor(template), "1210012")
		template = 'im<:"Wiki":>.'
		self.assertEquals(templateProcessor(template), "imWiki.")

		template = '<:name:>'
		name = 'imWiki'
		localNS = locals()
		self.assertEquals(templateProcessor(template, {}, localNS), "imWiki")

	def test_Statements(self):
		template = '''<:#statements
for i in range(10):
	print name
:>'''
		name = 'imWiki'
		localNS = locals()
		self.assertEquals(templateProcessor(template, {}, localNS), "imWiki\n"*10)

	def test_Statements(self):
		template = '''<:#statements
teul = """Hello
World"""
print teul
:>'''
		self.assertEquals(templateProcessor(template, {}, {}), "Hello\nWorld\n")

	def test_Include(self):
		setTemplateFolder(self.testFolder)

		fileName = 'header.teul'
		path = os.path.join(self.testFolder, fileName)
		s = '''<:imwiki:>'''
		file(path, 'wb').write(s)

		template = '''<:#include %s:>''' % fileName
		imwiki = 'good~'
		localNS = locals()
		self.assertEquals(templateProcessor(template, {}, localNS), "good~")

if __name__ == '__main__':
	unittest.main(argv=('', '-v'))
