# -*- coding: utf8 -*-
import unittest
import sys
sys.path.append('..') 

from imFormatter.HTMLFormatter import HTMLFormatter
import config

class HTMLFormatterTestCase(unittest.TestCase):
	def test_TableFormatting(self):
		formatter = HTMLFormatter()
		self.assertEquals(formatter.table([['1']]), '<table><tr><td>1</td></tr></table>')
		self.assertEquals(formatter.table([['1', '2']]), '<table><tr><td>1</td><td>2</td></tr></table>')
		self.assertEquals(formatter.table([['1'], ['2']]), '<table><tr><td>1</td></tr><tr><td>2</td></tr></table>')

		self.assertEquals(formatter.table([[('left', '1')], ['2']]), '<table><tr><td align="left">1</td></tr><tr><td>2</td></tr></table>')
		self.assertEquals(formatter.table([[('right', '1')], ['2']]), '<table><tr><td align="right">1</td></tr><tr><td>2</td></tr></table>')
		self.assertEquals(formatter.table([[('center', '1')], ['2']]), '<table><tr><td align="center">1</td></tr><tr><td>2</td></tr></table>')

	def test_IndentFormatting(self):
		formatter = HTMLFormatter()
		self.assertEquals(formatter.indent([(1, None, 'line1')]), '<div class="imIndent">line1</div>')

		self.assertEquals(formatter.indent([(1, None, 'line1'), (2, None, 'line2'), (-1, None, 'line3')]), '<div class="imIndent">line1<div class="imIndent"><div class="imIndent">line2</div>line3</div></div>')

		self.assertEquals(formatter.indent([(1, None, 'line1\n'), (-1, None, '\n'), (1, None, 'line2\n')]), '<div class="imIndent">line1\n</div>\n<div class="imIndent">line2\n</div>')

	def test_List(self):
		formatter = HTMLFormatter()
		L = [(1, '*', 'line1')]
		self.assertEquals(formatter.indent(L), '<ul><li>line1</li></ul>')

		L = [(1, '1', 'line1')]
		self.assertEquals(formatter.indent(L), '<ol type="1"><li>line1</li></ol>')

		L = [(1, '1', 'line1'), (0, 'a', 'line2')]
		self.assertEquals(formatter.indent(L), '<ol type="1"><li>line1</li><li>line2</li></ol>')

		L = [(1, 'a', 'line1'), (1, '*', 'line2'), (1, 'i', 'line3'), (-2, 'A', 'line4')]
		self.assertEquals(formatter.indent(L), '<ol type="a"><li>line1<ul><li>line2<ol type="i"><li>line3</li></ol></li></ul></li><li>line4</li></ol>')

		L = [(1, '*', 'line1'), (1, 'a', 'line2'), (-1, 'i', 'line3')]
		self.assertEquals(formatter.indent(L), '<ul><li>line1<ol type="a"><li>line2</li></ol></li><li>line3</li></ul>')

		L = [(1, '*', 'line1'), (1, 'a', 'line2'), (1, 'i', 'line3'), (-2, '*', 'line4'), (1, 'I', 'line5')]
		self.assertEquals(formatter.indent(L), '<ul><li>line1<ol type="a"><li>line2<ol type="i"><li>line3</li></ol></li></ol></li><li>line4<ol type="I"><li>line5</li></ol></li></ul>')

		L = [(1, '*', 'line1'), (1, '*', 'line2'), (-1, '*', 'line3'), (0, '*', 'line4')]
		self.assertEquals(formatter.indent(L), '<ul><li>line1<ul><li>line2</li></ul></li><li>line3</li><li>line4</li></ul>')

	def test_indent(self):
		formatter = HTMLFormatter()

		self.assertEquals(formatter.indent([(1, None, 'line')]), '<div class="imIndent">line</div>')

		self.assertEquals(formatter.indent([(1, '*', 'line')]), '<ul><li>line</li></ul>')

		self.assertEquals(formatter.indent([(1, '*', 'line'), (0, None, 'line2')]), '<ul><li>lineline2</li></ul>')

		self.assertEquals(formatter.indent([(1, '*', 'line'), (1, None, 'line2')]), '<ul><li>line<div class="imIndent">line2</div></li></ul>')

		self.assertEquals(formatter.indent([(1, '*', 'line'), (1, None, 'line2'), (1, '1', 'line3')]), '<ul><li>line<div class="imIndent">line2<ol type="1"><li>line3</li></ol></div></li></ul>')


	def test_variable(self):
		D = { 
			'pre':'pre',
			'tt':'tt', 
			'strong':'strong',
			'em':'em', 
			'strike':'strike',
			'underline':'u',
			'listitem':'li',
			'superscript':'sup',
			'subscript':'sub',
		}	 
		formatter = HTMLFormatter()
		for methodName, mark in D.items():
			self.assertEquals(getattr(formatter, methodName)('text'), '<%s>text</%s>' % (mark, mark))
	def test_smiley(self):
		formatter = HTMLFormatter()
		self.assertEquals(formatter.smiley(':)'), '<img src="%s/img/%s" alt="%s" />' % (config.urlPrefix, config.smileyDict[':)'], ':)'))
	def test_Entity(self):
		formatter = HTMLFormatter()
		self.assertEquals(formatter.entity('>'), '&gt;')
		self.assertEquals(formatter.entity('&'), '&amp;')
		self.assertEquals(formatter.entity('<'), '&lt;')
		self.assertEquals(formatter.entity('NOTEXIST'), '')


if __name__ == '__main__':
	unittest.main(argv=('', '-v'))
