# -*- coding: utf8 -*-
import unittest 

import Repository
import Category
import Page
import Request
from Tag import *

class TagTestCase(unittest.TestCase):
	def setUp(self):
		Repository.createRepository()
		self.category = Category.Category()
		self.request = Request.Request()
		self.orgActionPath = self.request.actionPath
		self.request.actionPath = 'imwiki.cgi'
		
	def tearDown(self):
		Repository.removeRepository()
		self.request.actionPath = self.orgActionPath

	def test_Init(self):
		tagName = 'Animal'
		tagObj = Tag(tagName)
		self.assert_(tagObj.tagPath)
		self.assertEquals(tagObj.getPagePathList(), [])

		path = os.path.join(Repository.getTagRepository(), utils.encode(tagName))
		file(path, 'wb').write(path)
		self.assert_(tagObj.tagPath)
		self.assertEquals(tagObj.getPagePathList(), [path])

		path = os.path.join(Repository.getTagRepository(), utils.encode(tagName))

	def test_Append(self):
		tagName = 'Animal'
		tagObj = Tag(tagName)
		self.assertEquals(tagObj.getPagePathList(), [])

		page = Page.createPage(self.category, 'hamtori', '')
		tagObj.append(page)
		self.assertEquals(tagObj.getPagePathList(), ['hamtori'])
		tagObj = Tag(tagName)
		self.assertEquals(tagObj.getPagePathList(), ['hamtori'])

		page2 = Page.createPage(self.category, 'hamsooni', '')
		tagObj.append(page2)
		self.assertEquals(tagObj.getPagePathList(), ['hamtori', 'hamsooni'])
		tagObj = Tag(tagName)
		self.assertEquals(tagObj.getPagePathList(), ['hamtori', 'hamsooni'])

		tagObj.append(page) # 이미 목록에 있는걸 추가
		self.assertEquals(tagObj.getPagePathList(), ['hamtori', 'hamsooni'])
		tagObj = Tag(tagName)
		self.assertEquals(tagObj.getPagePathList(), ['hamtori', 'hamsooni'])
	def test_Remove(self):
		tagName = 'Animal'
		tagObj = Tag(tagName)
		self.assertEquals(tagObj.getPagePathList(), [])

		page = Page.createPage(self.category, 'hamtori', '')
		tagObj.append(page)
		tagObj = Tag(tagName)
		self.assertEquals(tagObj.getPagePathList(), ['hamtori'])

		page2 = Page.createPage(self.category, 'ham', '')
		self.assertEquals(tagObj.remove(page2), False)  
		self.assertEquals(tagObj.getPagePathList(), ['hamtori'])

		self.assert_(tagObj.remove(page))
		self.assertEquals(tagObj.getPagePathList(), [])
		tagObj = Tag(tagName)
		self.assertEquals(tagObj.getPagePathList(), [])

	def test_toLink(self):
		tagName = 'Animal'
		tagObj = Tag(tagName)
		self.assertEquals(tagObj.toLink(), '<a href="imwiki.cgi?action=pageListInTag&amp;tagName=Animal">Animal</a>') 

if __name__ == '__main__':
	unittest.main(argv=('', '-v'))
