# -*- coding: utf8 -*-
import os
import unittest

import Repository
from Logger import *

class LoggerTestCase(unittest.TestCase):
	def setUp(self):
		Repository.createRepository()
	def tearDown(self):
		Repository.removeRepository()
	def test_Log(self):
		logDir = Repository.getLogRepository()
		logPath = os.path.join(logDir, 'editLog')
		self.failIf(os.path.exists(logPath))
		log('editLog', 1,2,3)
		self.assert_(os.path.exists(logPath))
		self.assertEquals(file(logPath, 'rb').read(), '\n1\t2\t3')
		log('editLog', 4,5,6)
		self.assertEquals(file(logPath, 'rb').read(), '\n1\t2\t3\n4\t5\t6')

	def test_EditLog(self):
		logDir = Repository.getLogRepository()
		logPath = os.path.join(logDir, editLogFile)
		self.failIf(os.path.exists(logPath))
		editLog(1,2,3)
		self.assert_(os.path.exists(logPath))
		self.assertEquals(file(logPath, 'rb').read(), '\n1\t2\t3')
		editLog(4,5,6)
		self.assertEquals(file(logPath, 'rb').read(), '\n1\t2\t3\n4\t5\t6')

	def test_GetRawLines_NotExist(self):	
		Repository.removeRepository()
		self.assertEquals(getRawLines(editLogFile), [])
		Repository.createRepository()

	def test_GetRawLines(self):
		for i in range(10):
			editLog('line %s' % i)

		self.assertEquals(getRawLines(editLogFile), ['line %s' % no for no in range(10)]) 

		self.assertEquals(getRawLines(editLogFile, 7), ['line 9']) 
		self.assertEquals(getRawLines(editLogFile, 7, True), ['line 8']) 

		self.assertEquals(getRawLines(editLogFile, 9), ['line 9']) 

		self.assertEquals(getRawLines(editLogFile, 8, True), ['line 8']) 
		self.assertEquals(getRawLines(editLogFile, _continue=True), ['line %s' % no for no in range(8)]) 
		self.assertEquals(getRawLines(editLogFile, 8, True), []) 

if __name__ == '__main__':
	unittest.main(argv=('', '-v'))
