# -*- coding: utf8 -*-
import unittest
from Permission import *
import Repository
import Page 
import Category

class PermFieldTestCase(unittest.TestCase):
	def test_PermField(self):
		permField = PermField('11')
		self.assertEquals(permField.read, True)
		self.assertEquals(permField.write, True)
		permField = PermField('00')
		self.assertEquals(permField.read, False)
		self.assertEquals(permField.write, False)
		permField = PermField('01')
		self.assertEquals(permField.read, False)
		self.assertEquals(permField.write, True)

	def test_PermFieldToBoolList(self):
		permField = PermField('11')
		self.assertEquals(permField.toBoolList(), [True, True])
		permField = PermField('10')
		self.assertEquals(permField.toBoolList(), [True, False])
		permField = PermField('01')
		self.assertEquals(permField.toBoolList(), [False, True])

	def test_Equals(self):
		perm1 = PermField('11')
		perm2 = PermField('11')
		self.assertEquals(perm1, perm2)

		perm2 = PermField('00')
		self.assertNotEquals(perm1, perm2)

		self.assertEquals(perm1, '11')
		self.assertEquals('11', perm1)

		self.assertNotEquals(perm1, '01')
		self.assertNotEquals('01', perm1)


class PermissionTestCase(unittest.TestCase):
	def test_Permission(self):
		perm = Permission('111111')
		self.assert_(isinstance(perm['member'], PermField))
		self.assert_(isinstance(perm['group'], PermField))
		self.assert_(isinstance(perm['anony'], PermField))
		self.assert_(perm['anony'].write, True)

	def test_PermToStrong(self):
		perm = Permission('111111')
		self.assert_('%s' % perm, '111111')

		perm = Permission('011110')
		self.assert_('%s' % perm, '011110')
		perm = Permission('011110')
		self.assert_(perm.toString(), '011110')

	def test_PermToBoolList(self):
		perm = Permission('111111')
		self.assert_(perm.toBoolList(), [True*6]) 
		perm = Permission('111000')
		self.assert_(perm.toBoolList(), [True*3, False*3]) 

	def test_Equals(self):
		perms1 = Permission('111111')
		perms2 = Permission('111111')
		self.assertEquals(perms1, perms2)
		self.assertEquals(perms1, '111111')
		self.assertEquals('111111', perms2)

class MockUser:
	def __init__(self, uid, uType):
		self.uid = uid
		self.uType = uType
class PermissionCheckerTestCase(unittest.TestCase):
	def setUp(self):
		Repository.createRepository()
	def tearDown(self):
		Repository.removeRepository()
	def test_NotImplementError(self):
		self.assertRaises(NotImplementError, PermissionChecker)
	def test_CategoryPermCheck(self):
		category = Category.Category()
		self.assertEquals(category.permCheck(MockUser('jangc', 'member'), 'read'), True)
		
		category.editAttr('110111', ['jangc'])
		self.assertEquals(category.permCheck(MockUser('jangc', 'member'), 'read'), False)
	def test_PagePermCheck(self):
		category = Category.Category()
		page = Page.createPage(category, 'title', 'text')
		self.assertEquals(page.permCheck(MockUser('jangc', 'member'), 'read'), True)
		page.editPerms('011111')
		self.assertEquals(page.permCheck(MockUser('jangc', 'member'), 'read'), False)

		category._editGroup(['jangc'])
		page.editPerms('110111')
		self.assertEquals(page.permCheck(MockUser('jangc', 'member'), 'read'), False)
	def test_RevisionPermCheck(self):
		# TODO 추가해야하는데 귀찮다..
		pass

if __name__ == '__main__':
	unittest.main(argv=('', '-v'))
