# -*- coding: utf8 -*-
import unittest
import time
import sys
sys.path.append('..')
import os
import Repository
import Logger

from imMacro.RecentChanges import *

class RecentChangesTestCase(unittest.TestCase):
	def setUp(self):
		Repository.createRepository()
		aHour = 3600
		aDay = 86400	
		writedTime = 1176082856.67
		Logger.editLog('do_edit', '', 'FrontPage', writedTime-aDay, 'jangc')
		Logger.editLog('do_edit', '', 'FrontPage', writedTime-aHour, 'jangc')
		Logger.editLog('do_edit', '', 'FrontPage', writedTime, 'jangc')

		Logger.editLog('do_edit', 'Seed', 'FrontPage', writedTime+aDay, 'jangc')

		Logger.editLog('do_edit', 'Seed', 'Help', writedTime+aDay+aHour, 'jangc')
		Logger.editLog('do_edit', '_ED_95_9C_EA_B8_80', '_ED_8E_98_EC_9D_B4_EC_A7_80', writedTime+aDay+aHour+10, 'jangc')

	def tearDown(self):
		Repository.removeRepository()
		os.environ['SCRIPT_NAME'] = ''
	def test_toDate(self):
		writedTime = '1176082856.67'
		self.assertEquals(makeDate(writedTime), 1176044400)

		writedTime = '1175993424.23'
		self.assertEquals(makeDate(writedTime), 1175958000)

	def test_Parse(self):
		self.assertEquals(parse(), {1176130800: [['do_edit', '한글', '페이지', '1176172866.67', 'jangc'], ['do_edit', 'Seed', 'Help', '1176172856.67', 'jangc'], ['do_edit', 'Seed', 'FrontPage', '1176169256.67', 'jangc']], 1176044400: [['do_edit', '', 'FrontPage', '1176082856.67', 'jangc']]})

		config.recentChangesMaxItemCount = 1
		self.assertEquals(parse(), {1176130800: [['do_edit', '한글', '페이지', '1176172866.67', 'jangc']]})

	def test_Parse_MaxCount(self):
		org = config.recentChangesMaxItemCount
		config.recentChangesMaxItemCount = 1
		self.assertEquals(parse(), {1176130800: [['do_edit', '한글', '페이지', '1176172866.67', 'jangc']]})

		config.recentChangesMaxItemCount = 2
		self.assertEquals(parse(), {1176130800: [['do_edit', '한글', '페이지', '1176172866.67', 'jangc'], ['do_edit', 'Seed', 'Help', '1176172856.67', 'jangc']]})

		config.recentChangesMaxItemCount = org

	def test_Parse_ChangeSize(self):
		self.assertEquals(parse(size=1), {})
		self.assertEquals(parse(size=100), {1176130800: [['do_edit', '한글', '페이지', '1176172866.67', 'jangc']]})

	def test_Parse_SizeAndCount(self):
		for i in range(1000):
			Logger.editLog('do_edit', 'Seed', 'Help', 1176130870, 'jangc')
		config.recentChangesMaxItemCount = 2
		self.assertEquals(parse(maxItemCount=2, size=100), {1176130800: [['do_edit', 'Seed', 'Help', '1176130870', 'jangc'], ['do_edit', '한글', '페이지', '1176172866.67', 'jangc']]})

	def test_CategoryPath(self):
		os.environ['PATH_INFO'] = '/Seed/FindPage'
		request = Request.Request()
		request.init()
		self.assertEquals(parse(), {1176130800: [['do_edit', 'Seed', 'Help', '1176172856.67', 'jangc'], ['do_edit', 'Seed', 'FrontPage', '1176169256.67', 'jangc']]})
		os.environ['PATH_INFO'] = '/SomePageInRoot'
		request.init()
		
		self.assertEquals(parse(), {1176130800: [['do_edit', '한글', '페이지', '1176172866.67', 'jangc'], ['do_edit', 'Seed', 'Help', '1176172856.67', 'jangc'], ['do_edit', 'Seed', 'FrontPage', '1176169256.67', 'jangc']], 1176044400: [['do_edit', '', 'FrontPage', '1176082856.67', 'jangc']]})
	def test_Run(self):
		self.assert_(run())

if __name__ == '__main__':
	unittest.main(argv=('', '-v'))
