# -*- coding: utf8 -*-
import unittest
import sys
sys.path.append('..')

from imParser import WikiParser

class IndentParserTestCase(unittest.TestCase):
	def test_GetType(self):
		parser = WikiParser.IndentParser()
		self.assertEquals(parser.getType(' * line'), '*')
		self.assertEquals(parser.getType(' 1. line'), '1')
		self.assertEquals(parser.getType(' line'), None)
	def test_Split(self):
		parser = WikiParser.IndentParser()
		self.assertEquals(parser.split(' line'), (1, None, 'line'))
		self.assertEquals(parser.split(' * line'), (1, '*', 'line'))
		self.assertEquals(parser.split('  * line'), (2, '*', 'line'))
		self.assertEquals(parser.split('  1. line'), (2, '1', 'line'))
	def test_joinToPrevContents(self):
		parser = WikiParser.IndentParser()
		parser.feed(' test')
		self.assertEquals(parser.parse(), [(1, None, 'test\n')])
		parser.joinToPrevContents('line2')
		self.assertEquals(parser.result, [(1, None, 'test\nline2\n')])

	def test_Parse(self):
		parser = WikiParser.IndentParser()
		parser.feed(' line')
		self.assertEquals(parser.parse(), [(1, None, 'line\n')])

		parser.feed(' * line')
		self.assertEquals(parser.parse(), [(1, '*', 'line\n')])

	def test_Parse_SameLevel(self):
		parser = WikiParser.IndentParser()

		parser.feed(' * line\n line2')
		self.assertEquals(parser.parse(), [(1, '*', 'line\nline2\n')])
		parser.feed(' * line\n line2\n line3')
		self.assertEquals(parser.parse(), [(1, '*', 'line\nline2\nline3\n')])

		parser.feed(' * line\n * line2')
		self.assertEquals(parser.parse(), [(1, '*', 'line\n'), (0, '*', 'line2\n')])
	def test_Parse_StepUpAndSameLevel(self):
		parser = WikiParser.IndentParser()

		parser.feed(' * line\n  * line2\n  line3')
		self.assertEquals(parser.parse(), [(1, '*', 'line\n'), (1, '*', 'line2\nline3\n')])

	def test_Parse_StepDownAndSameLevel(self):
		parser = WikiParser.IndentParser()
		parser.feed(' * line\n  * line2\n line3\n line4')
		self.assertEquals(parser.parse(), [(1, '*', 'line\n'), (1, '*', 'line2\n'), (-1, None, 'line3\nline4\n')])

	def test_Parse_StepUp(self):
		parser = WikiParser.IndentParser()

		parser.feed(' * line\n  line2')
		self.assertEquals(parser.parse(), [(1, '*', 'line\n'), (1, None, 'line2\n')])

		parser.feed(' * line\n  line2\n   line3')
		self.assertEquals(parser.parse(), [(1, '*', 'line\n'), (1, None, 'line2\n'), (1, None, 'line3\n')])

	def test_Parse_StepDown(self):
		parser = WikiParser.IndentParser()

		parser.feed(' * line\n  line2\n line3')
		self.assertEquals(parser.parse(), [(1, '*', 'line\n'), (1, None, 'line2\n'), (-1, None, 'line3\n')])

		parser.feed(' * line\n  line2\n   line3\n line4')
		self.assertEquals(parser.parse(), [(1, '*', 'line\n'), (1, None, 'line2\n'), (1, None, 'line3\n'), (-2, None, 'line4\n')])


	def test_indent(self):
		indentParser = WikiParser.IndentParser()
		
		indentParser.feed(""" line1""")
		self.assertEquals(indentParser.parse(), [(1, None, 'line1\n')])

		indentParser.feed(""" line1\n""")
		self.assertEquals(indentParser.parse(), [(1, None, 'line1\n')])

		indentParser.feed(""" line1\n line2""")
		self.assertEquals(indentParser.parse(), [(1, None, 'line1\nline2\n')])

		indentParser.feed(""" line1\n  line2\n   line3""")
		self.assertEquals(indentParser.parse(), [(1, None, 'line1\n'), (1, None, 'line2\n'), (1, None, 'line3\n')])

		indentParser.feed(""" line1\n  line2\n line3""")
		self.assertEquals(indentParser.parse(), [(1, None, 'line1\n'), (1, None, 'line2\n'), (-1, None, 'line3\n')])

		indentParser.feed(""" line1\n  line2\n   line3\n line4""")
		self.assertEquals(indentParser.parse(), [(1, None, 'line1\n'), (1, None, 'line2\n'), (1, None, 'line3\n'), (-2, None, 'line4\n')])

		indentParser.feed(""" line1\n  line2\n   line3\n line4\n   line5""")
		self.assertEquals(indentParser.parse(), [(1, None, 'line1\n'), (1, None, 'line2\n'), (1, None, 'line3\n'), (-2, None, 'line4\n'), (1, None, 'line5\n')])

		indentParser.feed(""" line1\n line2\n  line3\n  line4""")
		self.assertEquals(indentParser.parse(), [(1, None, 'line1\nline2\n'), (1, None, 'line3\nline4\n')])

		indentParser.feed('    \n')
		self.assertEquals(indentParser.parse(), [(0, None, '\n')])

		indentParser.feed('    \n \n')
		self.assertEquals(indentParser.parse(), [(0, None, '\n\n')])

		indentParser.feed('    \n line1')
		self.assertEquals(indentParser.parse(), [(0, None, '\n'), (1, None, 'line1\n')])

		indentParser.feed('    \n * line1')
		self.assertEquals(indentParser.parse(), [(0, None, '\n'), (1, '*', 'line1\n')])



	def test_parsing2(self): # 여러개의 \n 처리.
		indentParser = WikiParser.IndentParser()
		indentParser.feed(""" line1\n line2\n\n\n line3""")
		self.assertEquals(indentParser.parse(), [(1, None, 'line1\nline2\n\n\nline3\n')])
		indentParser.feed(""" line1\n line2\n \n\n line3""")
		self.assertEquals(indentParser.parse(), [(1, None, 'line1\nline2\n\n\nline3\n')])  # caution
	def test_unbalance(self): # caution
		indentParser = WikiParser.IndentParser()
		indentParser.feed("""   line1\n  line2\n line3\n  line4""")
		self.assertEquals(indentParser.parse(), [(1, None, 'line1\n'), (0, None, 'line2\n'), (1, None, 'line3\n'), (1, None, 'line4\n')])

class TableParserTestCase(unittest.TestCase):
	def test_multiCellParsing(self):
		tableParser = WikiParser.TableParser()
		
		tableParser.feed("""|| im wiki ||""")
		self.assertEquals(tableParser.parse(), [[' im wiki ']])

		tableParser.feed("""|| im wiki ||   """)
		self.assertEquals(tableParser.parse(), [[' im wiki ']])

		tableParser.feed("""|| im wiki || second cell ||""")
		self.assertEquals(tableParser.parse(), [[' im wiki ', ' second cell ']])

		tableParser.feed("""|| im wiki || second cell ||
|| egg || kimbab ||""")
		self.assertEquals(tableParser.parse(), [[' im wiki ', ' second cell '], [' egg ', ' kimbab ']])
	def test_align(self):
		tableParser = WikiParser.TableParser()
		
		tableParser.feed("""||< im wiki ||""")
		self.assertEquals(tableParser.parse(), [[('left', ' im wiki ')]])

		tableParser.feed("""||> im wiki ||""")
		self.assertEquals(tableParser.parse(), [[('right', ' im wiki ')]])

		tableParser.feed("""||<> im wiki ||""")
		self.assertEquals(tableParser.parse(), [[('center', ' im wiki ')]])

		tableParser.feed("""||>< im wiki ||""")
		self.assertEquals(tableParser.parse(), [[('center', ' im wiki ')]])

class wikiTestCase(unittest.TestCase):
	def test_feed(self):
                rawText = 'imwiki'
		parser = WikiParser.Parser(WikiParser.EchoFormatter())
		parser.feed(rawText)
		self.assertEquals(parser.rawText, rawText)
		
	def test_Strong(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())

		parser.feed("'''imwiki'''")
		self.assertEquals(parser.parse(), "<strong>imwiki</strong>")

		parser.feed("im'''wiki'''")
		self.assertEquals(parser.parse(), "im<strong>wiki</strong>")

		parser.feed("''''imwiki'''")
		self.assertEquals(parser.parse(), "'<strong>imwiki</strong>")

		parser.feed("''''''imwiki'''")
		self.assertEquals(parser.parse(), "'''<strong>imwiki</strong>")

		parser.feed("''''''im'wiki'''")
		self.assertEquals(parser.parse(), "'''<strong>im'wiki</strong>")

		parser.feed("'''im'\nwiki'''")
		self.assertEquals(parser.parse(), "<strong>im'\nwiki</strong>")

		parser.feed("''' im'wiki'''")
		self.assertEquals(parser.parse(), "''' im'wiki'''")

	def test_Em(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())

		parser.feed("''imwiki''")
		self.assertEquals(parser.parse(), "<em>imwiki</em>")

		parser.feed("''imwiki'''")
		self.assertEquals(parser.parse(), "<em>imwiki</em>'")

	def test_StrongEm(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())

		parser.feed("'''''imwiki'''''")
		self.assertEquals(parser.parse(), "<strongEm>imwiki</strongEm>")

		parser.feed("'''''imwiki'''''kiwi'''''")
		self.assertEquals(parser.parse(), "<strongEm>imwiki</strongEm>kiwi'''''")

		parser.feed("''im'''''''imwiki'''''kiwi'''")
		self.assertEquals(parser.parse(), "<em>im</em><strongEm>imwiki</strongEm>kiwi'''")

		parser.feed("''im\nkiwi''")
		self.assertEquals(parser.parse(), "<em>im\nkiwi</em>")
	def test_Superscript(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())

		parser.feed('^^test^^')
		self.assertEquals(parser.parse(), "<superscript>test</superscript>")

		parser.feed('^^t^est^^')
		self.assertEquals(parser.parse(), "<superscript>t^est</superscript>")

		parser.feed('^^^test^^')
		self.assertEquals(parser.parse(), "^<superscript>test</superscript>")

		parser.feed('^^^^test^^')
		self.assertEquals(parser.parse(), "^^<superscript>test</superscript>")
	def test_Subscript(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())

		parser.feed(',,test,,')
		self.assertEquals(parser.parse(), "<subscript>test</subscript>")

		parser.feed(',,t,est,,')
		self.assertEquals(parser.parse(), "<subscript>t,est</subscript>")

		parser.feed(',,,test,,')
		self.assertEquals(parser.parse(), ",<subscript>test</subscript>")

		parser.feed(',,,,test,,')
		self.assertEquals(parser.parse(), ",,<subscript>test</subscript>")

	def test_Strike(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())

		parser.feed("~~imwiki~~")
		self.assertEquals(parser.parse(), "<strike>imwiki</strike>")

		parser.feed("~~ imwiki~~")
		self.assertEquals(parser.parse(), "~~ imwiki~~")

		parser.feed("~~~imwiki~~~")
		self.assertEquals(parser.parse(), "~<strike>imwiki</strike>~")

		parser.feed("~~im\nwiki~~")
		self.assertEquals(parser.parse(), "<strike>im\nwiki</strike>")

	
	def test_Horizon(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())

		parser.feed("----")
		self.assertEquals(parser.parse(), "<horizon>----</horizon>")
		
		parser.feed("-------")
		self.assertEquals(parser.parse(),  "<horizon>-------</horizon>")

		parser.feed("---- ")
		self.assertEquals(parser.parse(), "---- ")

		parser.feed("----\n")
		self.assertEquals(parser.parse(),  "<horizon>----\n</horizon>")
	
	def test_Underline(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())

		parser.feed("__imwiki__")
		self.assertEquals(parser.parse(), "<underline>imwiki</underline>")

		parser.feed("___imwiki___")
		self.assertEquals(parser.parse(), "_<underline>imwiki</underline>_")

		parser.feed("__im\nwiki__")
		self.assertEquals(parser.parse(), "<underline>im\nwiki</underline>")


	def test_Heading(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())

		parser.feed("= imwiki =")
		self.assertEquals(parser.parse(), "<heading>imwiki : 1</heading>")

		parser.feed("===== imwiki ====")
		self.assertEquals(parser.parse(), "===== imwiki ====")

		parser.feed("===== imwiki =====")
		self.assertEquals(parser.parse(), "<heading>imwiki : 5</heading>")

		parser.feed("======= imwiki =======")
		self.assertEquals(parser.parse(), "======= imwiki =======")

		parser.feed("x== imwiki ==")
		self.assertEquals(parser.parse(), "x== imwiki ==")
		
		parser.feed("== imwiki == x")
		self.assertEquals(parser.parse(), "== imwiki == x")

		parser.feed("=== imwiki ===\n") 
		self.assertEquals(parser.parse(), "<heading>imwiki : 3</heading>")

		parser.feed("=== imwiki ===\n\n\n") 
		self.assertEquals(parser.parse(), "<heading>imwiki : 3</heading><br /><br />")


		parser.feed("=== im\nwiki ===") 
		self.assertEquals(parser.parse(), "=== im<br />wiki ===")

		parser.feed(" === imwiki ===") 
		self.assertEquals(parser.parse(), "<indent>[(1, None, '<heading>imwiki : 3</heading>')]</indent>")


		# depth test
		parser.feed("""
'''I'm very strong!!!!'''
=== I have three head! ===
""")
		self.assertEquals(parser.parse(), """<br /><strong>I'm very strong!!!!</strong><br /><heading>I have three head! : 3</heading>""")

	def test_deepReplace(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())

		parser.feed("'''__Im Wiki__'''")
		self.assertEquals(parser.parse(), "<strong><underline>Im Wiki</underline></strong>")
		parser.feed("'''__Im Wiki'''__")
		self.assertEquals(parser.parse(), "<strong>__Im Wiki</strong>__")
		
	def test_Pre(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())

		parser.feed("""
{{{
== im ==
'''wiki'''
}}}
""")
		self.assertEquals(parser.parse(), """<br /><pre>
== im ==
'''wiki'''
</pre>""")

		parser.feed("""
{{{
== im ==
'''wiki'''
}}}
normal line
{{{
''egg''
}}}""")

		self.assertEquals(parser.parse(), """<br /><pre>
== im ==
'''wiki'''
</pre>normal line<br /><pre>
''egg''
</pre>""")

		# 예외의 경우..formatter.entity로 변경하는게 아니라 바로 바꿔버림 cgi.escape
		parser.feed("""{{{\n<html>\n}}}""")
		self.assertEquals(parser.parse(), """<pre>\n&lt;html&gt;\n</pre>""")
	
	def test_Table(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())

		parser.feed("|| egg ||")
		self.assertEquals(parser.parse(), """<table>[[' egg ']]</table>""")

		parser.feed("||> egg ||")
		self.assertEquals(parser.parse(), """<table>[[('right', ' egg ')]]</table>""")

		parser.feed("||<> egg ||")
		self.assertEquals(parser.parse(), """<table>[[('center', ' egg ')]]</table>""")

		parser.feed("|| [egg] ||")
		self.assertEquals(parser.parse(), "<table>[[' <internalLink> : egg : True</internalLink> ']]</table>")

		parser.feed("|| '''egg''' ||")
		self.assertEquals(parser.parse(), "<table>[[' <strong>egg</strong> ']]</table>")

		parser.feed("|| egg ||")
		self.assertEquals(parser.parse(), """<table>[[' egg ']]</table>""")

		parser.feed("|| egg || spam ||")
		self.assertEquals(parser.parse(), """<table>[[' egg ', ' spam ']]</table>""")

		parser.feed("|| egg || spam ||\n|| 2 || 3 ||")
		self.assertEquals(parser.parse(), """<table>[[' egg ', ' spam '], [' 2 ', ' 3 ']]</table>""")

		parser.feed("|| egg || spam ||\n||||||")
		self.assertEquals(parser.parse(), """<table>[[' egg ', ' spam '], ['', '']]</table>""")

		parser.feed("|| egg || spam ||\n||||")
		self.assertEquals(parser.parse(), """<table>[[' egg ', ' spam '], ['']]</table>""")

		parser.feed("""|| Python ||
 || egg ||
 || ham ||
""")
		self.assertEquals(parser.parse(), """<table>[[' Python ']]</table><indent>[(1, None, "<table>[[' egg '], [' ham ']]</table>")]</indent>""")

	def test_Indent(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())

		parser.feed("line1\n line2")
		self.assertEquals(parser.parse(), "line1<br /><indent>[(1, None, 'line2<br />')]</indent>")

		parser.feed("line1\n line2\n  line3")
		self.assertEquals(parser.parse(), "line1<br /><indent>[(1, None, 'line2<br />'), (1, None, 'line3<br />')]</indent>")

		parser.feed("line1\n line2\n  line3\n line4")
		self.assertEquals(parser.parse(), "line1<br /><indent>[(1, None, 'line2<br />'), (1, None, 'line3<br />'), (-1, None, 'line4<br />')]</indent>")

		parser.feed("line1\n line2\n  line3\n\nline4")
		self.assertEquals(parser.parse(), "line1<br /><indent>[(1, None, 'line2<br />'), (1, None, 'line3<br /><br />')]</indent>line4")

		parser.feed("line1\n line2\n  line3\nline4")
		self.assertEquals(parser.parse(), "line1<br /><indent>[(1, None, 'line2<br />'), (1, None, 'line3<br />')]</indent>line4")

		parser.feed(" '''line1'''")
		self.assertEquals(parser.parse(), """<indent>[(1, None, '<strong>line1</strong><br />')]</indent>""") 

		parser.feed(" line1\n * line2")
		self.assertEquals(parser.parse(), 
				"<indent>[(1, None, 'line1<br />'), (0, '*', 'line2<br />')]</indent>")

		parser.feed(""" line1\n line2\n\n line3""")
		self.assertEquals(parser.parse(), """<indent>[(1, None, 'line1<br />line2<br /><br />line3<br />')]</indent>""")  

	def test_List(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())

		parser.feed(" * line1")

		self.assertEquals(parser.parse(), "<indent>[(1, '*', 'line1<br />')]</indent>")

		parser.feed(" * line1\n * line2")
		self.assertEquals(parser.parse(), "<indent>[(1, '*', 'line1<br />'), (0, '*', 'line2<br />')]</indent>")

		parser.feed(" * line1\n * line2\nline3")
		self.assertEquals(parser.parse(), "<indent>[(1, '*', 'line1<br />'), (0, '*', 'line2<br />')]</indent>line3")

		parser.feed(" 1. line1\n 1. line2\nline3")
		self.assertEquals(parser.parse(), "<indent>[(1, '1', 'line1<br />'), (0, '1', 'line2<br />')]</indent>line3")

		parser.feed(" 1. line1\n 1. line2\n line3")
		self.assertEquals(parser.parse(), "<indent>[(1, '1', 'line1<br />'), (0, '1', 'line2<br />line3<br />')]</indent>")

		parser.feed(" * line1\n\n * line2")
		self.assertEquals(parser.parse(), "<indent>[(1, '*', 'line1<br /><br />'), (0, '*', 'line2<br />')]</indent>") 

		parser.feed(" 1. line1\n  1. line2\n 1. line3\n 1. line4")
		self.assertEquals(parser.parse(), "<indent>[(1, '1', 'line1<br />'), (1, '1', 'line2<br />'), (-1, '1', 'line3<br />'), (0, '1', 'line4<br />')]</indent>")
	def test_ListType(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())
		parser.feed(" * line1")
		self.assertEquals(parser.parse(), "<indent>[(1, '*', 'line1<br />')]</indent>")

		parser.feed(" 1. line1\n a. line2")
		self.assertEquals(parser.parse(), "<indent>[(1, '1', 'line1<br />'), (0, 'a', 'line2<br />')]</indent>")

		parser.feed(" a. line1\n 1. line2")
		self.assertEquals(parser.parse(), "<indent>[(1, 'a', 'line1<br />'), (0, '1', 'line2<br />')]</indent>")

	def test_Tt(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())
		parser.feed("Im{{{''Wiki''}}}.")
		self.assertEquals(parser.parse(), "Im<tt>''Wiki''</tt>.")
		parser.feed("Im{{{<''Wiki''>}}}.")
		self.assertEquals(parser.parse(), "Im<tt>&lt;''Wiki''&gt;</tt>.")

	def test_InternalLink(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())
		parser.feed("Im[Wiki].")
		self.assertEquals(parser.parse(), 'Im<internalLink> : Wiki : True</internalLink>.')

		parser.feed("Im[Wiki]da.")
		self.assertEquals(parser.parse(), 'Im<internalLink> : Wiki : True</internalLink>da.')


		parser.feed("Im[[Wiki]da.")
		self.assertEquals(parser.parse(), 'Im[<internalLink> : Wiki : True</internalLink>da.')

		parser.feed("Im[Seed/Wiki]da.")
		self.assertEquals(parser.parse(), 'Im<internalLink>Seed : Wiki : True</internalLink>da.')

		parser.feed("Im[Default/Seed/Wiki]da.")
		self.assertEquals(parser.parse(), 'Im<internalLink>Default/Seed : Wiki : True</internalLink>da.')

		parser.feed("Im[/Default]da.")
		#self.assertEquals(parser.parse(), 'Im<internalLink> : Default : False</internalLink>da.')
		self.assertEquals(parser.parse(), 'Im<internalLink> : Default : False : Default</internalLink>da.')

		parser.feed("Im[/Default/System]da.")
		#self.assertEquals(parser.parse(), 'Im<internalLink>/Default : System : False</internalLink>da.')
		self.assertEquals(parser.parse(), 'Im<internalLink>/Default : System : False : System</internalLink>da.')

		parser.feed("Im[http://jangc.net]da.")
		self.assertEquals(parser.parse(), 'Im<externalLink>http://jangc.net</externalLink>da.')

		parser.feed("Im[http://jangc.net JANGC Homepage]da.")
		self.assertEquals(parser.parse(), 'Im<externalLink>http://jangc.net : JANGC Homepage</externalLink>da.')

	def test_CamelCase(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())
		parser.feed("Im FrontPage.")
		self.assertEquals(parser.parse(), 'Im <internalLink> : FrontPage : True</internalLink>.')

		parser.feed("Im /FrontPage.")
		#self.assertEquals(parser.parse(), 'Im <internalLink> : FrontPage : False</internalLink>.')
		self.assertEquals(parser.parse(), 'Im <internalLink> : FrontPage : False : FrontPage</internalLink>.')

		parser.feed("Im CateGory/FrontPage.")
		self.assertEquals(parser.parse(), 'Im <internalLink>CateGory : FrontPage : True</internalLink>.')

		parser.feed("Im CateGory/SubCategory/FrontPage.")
		self.assertEquals(parser.parse(), 'Im <internalLink>CateGory/SubCategory : FrontPage : True</internalLink>.')

		parser.feed("Im /CateGory/SubCategory/FrontPage.") 
		self.assertEquals(parser.parse(), 'Im <internalLink>/CateGory/SubCategory : FrontPage : False : FrontPage</internalLink>.')

	def test_RelationalLink(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())

		parser.feed("<PageLink>")
		self.assertEquals(parser.parse(), '<internalLink> : PageLink : False : PageLink</internalLink>')

		parser.feed("</PageLink>")
		self.assertEquals(parser.parse(), '<internalLink> : PageLink : True</internalLink>')

		parser.feed("<Category/Category2/PageLink>")
		self.assertEquals(parser.parse(), '<internalLink>Category/Category2 : PageLink : False : PageLink</internalLink>')

		parser.feed("</Category/Category2/PageLink>")
		#self.assertEquals(parser.parse(), '<internalLink>/Category/Category2 : PageLink : True : PageLink</internalLink>')
		self.assertEquals(parser.parse(), '<internalLink>/Category/Category2 : PageLink : True</internalLink>')

	def test_ExternalLink(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())
		parser.feed("http://jangc.net")
		self.assertEquals(parser.parse(), '<externalLink>http://jangc.net</externalLink>')
		parser.feed("im http://jangc.net?action=view da")
		self.assertEquals(parser.parse(), 'im <externalLink>http://jangc.net?action=view</externalLink> da')

	def test_Macro(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())
		parser.feed("Im[[Wiki]].")
		self.assertEquals(parser.parse(), "Im<macro>Wiki</macro>.")

		parser.feed(" [[Wiki]]")
		self.assertEquals(parser.parse(), "<indent>[(1, None, '<macro>Wiki</macro><br />')]</indent>")

		parser.feed(" * [[Wiki]]")
		self.assertEquals(parser.parse(), "<indent>[(1, '*', '<macro>Wiki</macro><br />')]</indent>")

		parser.feed("||[[Wiki]]||")
		self.assertEquals(parser.parse(), "<table>[['<macro>Wiki</macro>']]</table>")


	def test_Smiley(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())
		parser.feed("Im happy! :)")
		self.assertEquals(parser.parse(), "Im happy! <smiley>:)</smiley>")

		parser.feed("Im happy! B)")
		self.assertEquals(parser.parse(), "Im happy! <smiley>B)</smiley>")

		parser.feed("Im happy!:)")
		self.assertEquals(parser.parse(), "Im happy!<smiley>:)</smiley>")

		parser.feed("Im happy! :)\n")
		self.assertEquals(parser.parse(), "Im happy! <smiley>:)</smiley><br />")

		parser.feed("Im happy! :)ABCD....")
		self.assertEquals(parser.parse(), "Im happy! <smiley>:)</smiley>ABCD....")

		parser.feed(" * B) spam")
		self.assertEquals(parser.parse(), "<indent>[(1, '*', '<smiley>B)</smiley> spam<br />')]</indent>")

	def test_Blockquote(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())

		parser.feed('''"""line1"""''')
		self.assertEquals(parser.parse(), '<q>line1</q>')

		parser.feed('''"""\nline1\nline2\nline3\n"""''')
		self.assertEquals(parser.parse(), "<blockquote>line1<br />line2<br />line3</blockquote>")

		parser.feed('''"""\n''line1''\n"""''')
		self.assertEquals(parser.parse(), "<blockquote><em>line1</em></blockquote>")
		parser.feed(''' * line1
"""
 * line2
"""''')
		self.assertEquals(parser.parse(), "<indent>[(1, '*', 'line1<br />')]</indent><blockquote><indent>[(1, '*', 'line2<br />')]</indent></blockquote>")

	def test_InlineQuote(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())

		parser.feed('''"""line1"""''')
		self.assertEquals(parser.parse(), '<q>line1</q>')

		parser.feed('''Hello """line1""" World''')
		self.assertEquals(parser.parse(), 'Hello <q>line1</q> World')

	def test_Entity(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())
		parser.feed('''<html''')
		self.assertEquals(parser.parse(), '<entity><</entity>html')

		parser.feed('''<>html''')
		self.assertEquals(parser.parse(), '<entity><</entity><entity>></entity>html')


		parser.feed('''&html''')
		self.assertEquals(parser.parse(), '<entity>&</entity>html')
	def test_Attachment(self):
		parser = WikiParser.Parser(WikiParser.EchoFormatter())
		parser.feed('attachment:picture.png')
		self.assertEquals(parser.parse(), '<attachment>picture.png</attachment>')

		parser.feed('abc attachment:picture.png def')
		self.assertEquals(parser.parse(), 'abc <attachment>picture.png</attachment> def')

		parser.feed('Noattachment:picture.png')
		self.assertEquals(parser.parse(), 'Noattachment:picture.png')

		parser.feed('\nattachment:picture.png\n')
		self.assertEquals(parser.parse(), '<br /><attachment>picture.png</attachment><br />')

	def xtest_Attachment_Short(self): #FIXME
		parser = WikiParser.Parser(WikiParser.EchoFormatter())
		parser.feed('''@@picture.png@@''')
		self.assertEquals(parser.parse(), '<attachment>picture.png</attachment>')

if __name__ == '__main__':
	unittest.main(argv=('', '-v'))
