# -*- coding: utf8 -*-
import unittest

from macroProcessor import *

class MacroProcessorTestCase(unittest.TestCase):
	def setUp(self):
		setMacroPath('imMacro.')
	def tearDown(self):
		pass
	def test_extractArgs(self):
		macroStr = 'RecentChanges'
		self.assertEquals(extractArgs(macroStr), ['RecentChanges', None])

		macroStr = 'RecentChanges(Month'
		self.assertEquals(extractArgs(macroStr), ['RecentChanges', None])

		macroStr = 'RecentChanges(100)'
		self.assertEquals(extractArgs(macroStr), ['RecentChanges', ['100']])

		macroStr = 'RecentChanges(100, 7)'
		self.assertEquals(extractArgs(macroStr), ['RecentChanges', ['100', '7']])

		macroStr = 'RecentChanges(Today)'
		self.assertEquals(extractArgs(macroStr), ['RecentChanges', ['Today']])

		macroStr = 'RecentChanges(Month,    12)'
		self.assertEquals(extractArgs(macroStr), ['RecentChanges', ['Month', '12']])

		macroStr = 'RecentChanges(Month,    12)'
		self.assertEquals(extractArgs(macroStr), ['RecentChanges', ['Month', '12']])

		macroStr = 'RecentChanges(Month,    12)'
		self.assertEquals(extractArgs(macroStr), ['RecentChanges', ['Month', '12']])

	def test_extractArgs_Keyword(self):
		macroStr = 'RecentChanges(type=Month)'
		self.assertEquals(extractArgs(macroStr), ['RecentChanges', [('type', 'Month')]])

		macroStr = 'RecentChanges(1st, type=Month)'
		self.assertEquals(extractArgs(macroStr), ['RecentChanges', ['1st', ('type', 'Month')]])

		macroStr = 'RecentChanges(1st, type  =  Month)'
		self.assertEquals(extractArgs(macroStr), ['RecentChanges', ['1st', ('type', 'Month')]])

		macroStr = 'RecentChanges(1st, type=Month, 1)'
		self.assertEquals(extractArgs(macroStr), ['RecentChanges', None])


	def test_loadMacro(self):

		macroName = 'HelloWorld'
		macroModule = loadMacro(macroName)
		self.assertEquals(macroModule.__name__, 'imMacro.HelloWorld')

		macroName = 'NotExistMacro'
		macroModule = loadMacro(macroName)
		self.assertEquals(macroModule, None)
	
	def test_runMacro(self):
		macroName = 'NotExistMacroName'
		args = None
		self.assertEquals(runMacro(macroName, args), '[[%s]]' % macroName)

		macroName = 'HelloWorld'
		args = None
		self.assertEquals(runMacro(macroName, args), 'Hello, World!')

		macroName = 'HelloWorld'
		args = None
		self.assertEquals(runMacro(macroName, args), 'Hello, World!')

		macroName = 'HelloWorld'
		args = ['1', '2']
		self.assertEquals(runMacro(macroName, args), 'Hello, World! : 1 : 2')

if __name__ == '__main__':
	unittest.main(argv=('', '-v'))
