# -*- coding: utf8 -*-
import unittest
import shutil
import os

from utils import *

class UtilTestCase(unittest.TestCase):
	def test_encodePageName(self):
		self.assertEquals(encode('한글 페이지'), '_ED_95_9C_EA_B8_80_20_ED_8E_98_EC_9D_B4_EC_A7_80')
		self.assertEquals(encode('Page Name'), 'Page_20Name')
	def test_decodePageName(self):
		self.assertEquals(decode('Page_20Name'), 'Page Name')
		self.assertEquals(decode('_ED_95_9C_EA_B8_80_20_ED_8E_98_EC_9D_B4_EC_A7_80'), '한글 페이지')
	def test_nl2br(self):
		s = '\n' * 3
		self.assertEquals(nl2br(s), '<br />'*3)

	def test_isValidName(self):
		self.assertEquals(isValidName('Name'), True)
		self.failIf(isValidName('Name!'))
		self.assert_(isValidName('장'))

	def test_isValidName_WhiteSpace(self):
		self.assert_(isValidName('서울 유람 안내도'))
		self.failIf(isValidName('서울 유람 안내도', whitespaceCheck=True))

	def test_isValidName_LengthCheck(self):
		self.assert_(isValidName('A'*40))
		self.failIf(isValidName('A'*41))
		self.assert_(isValidName('A'))
		self.failIf(isValidName(''))

	def test_isValidName_LengthCheck_ChangeMinMax(self):
		self.failIf(isValidName('A'*11, maxLength=10))
		self.assert_(isValidName('A'*10, maxLength=10))

		self.assert_(isValidName('A'*10, minLength=10))
		self.failIf(isValidName('A'*9, minLength=10))

		self.assert_(isValidName('A'*10, minLength=10, maxLength=10))
		self.assert_(isValidName('A'*10, minLength=9, maxLength=10))
		self.assert_(isValidName('A'*10, minLength=10, maxLength=11))
	def test_makeStairLink(self):
		Request.Request().scriptName = 'imwiki.cgi'
		self.assertEquals(makeStairLink(''), '')
		self.assertEquals(makeStairLink('first'), '<a href="imwiki.cgi/first" title="first">first</a>')
		self.assertEquals(makeStairLink('first/second'), '<a href="imwiki.cgi/first" title="first">first</a> > <a href="imwiki.cgi/first/second" title="first/second">second</a>')

if __name__ == '__main__':
	unittest.main(argv=('','-v'))


